@@ -42,17 +42,13 @@
 // LibQApt includes
 #include <LibQApt/Backend>
 
-// Libmuon includes
-#include "MuonStrings.h"
-#include <ApplicationBackend/Application.h>
-#include <ApplicationBackend/ApplicationBackend.h>
+//libmuonapt includes
 #include <HistoryView/HistoryView.h>
-#include <resources/ResourcesModel.h>
-#include <QAptActions.h>
+#include "../libmuonapt/QAptActions.h"
 
-#ifdef ATTICA_ENABLED
-#include <KNSBackend/KNSBackend.h>
-#endif
+// Libmuon includes
+#include <resources/ResourcesModel.h>
+#include <MuonBackendsFactory.h>
 
 // Own includes
 #include "ApplicationLauncher.h"
@@ -60,6 +56,7 @@
 #include "AvailableView.h"
 #include "ProgressView.h"
 #include "ViewSwitcher.h"
+#include "LaunchListModel.h"
 #include "MuonInstallerSettings.h"
 
 enum ViewModelRole {
@@ -89,6 +86,7 @@ enum ViewType {
 MainWindow::MainWindow()
     : MuonMainWindow()
     , m_appBackend(nullptr)
+    , m_launches(nullptr)
     , m_launcherMessage(nullptr)
     , m_appLauncher(nullptr)
     , m_progressItem(nullptr)
@@ -113,7 +111,7 @@ void MainWindow::initGUI()
     // Set up the navigational sidebar on the right
     m_viewSwitcher = new ViewSwitcher(this);
     connect(m_viewSwitcher, SIGNAL(activated(QModelIndex)),
-           this, SLOT(changeView(QModelIndex)));
+            this, SLOT(changeView(QModelIndex)));
     m_mainWidget->addWidget(m_viewSwitcher);
 
     // Set up the main pane
@@ -141,8 +139,11 @@ void MainWindow::initGUI()
     m_viewModel = new QStandardItemModel(this);
     m_viewSwitcher->setModel(m_viewModel);
 
+    QAptActions *actions = QAptActions::self();
+    actions->setMainWindow(this);
+
     setupActions();
-    setupGUI((StandardWindowOption)(KXmlGuiWindow::Default & ~KXmlGuiWindow::StatusBar));
+    setupGUI(StandardWindowOption(KXmlGuiWindow::Default & ~KXmlGuiWindow::StatusBar));
 }
 
 void MainWindow::initObject()
@@ -153,33 +154,30 @@ void MainWindow::initObject()
     connect(resourcesModel, SIGNAL(transactionRemoved(Transaction*)),
             this, SLOT(transactionRemoved()));
 
-    // Create APT backend
-    m_appBackend = new ApplicationBackend(this);
-    m_actions->setBackend(m_appBackend->backend());
-    connect(m_appBackend, SIGNAL(backendReady()),
-            this, SLOT(populateViews()));
-    connect(m_appBackend, SIGNAL(reloadStarted()),
-            this, SLOT(removeProgressItem()));
-    connect(m_appBackend, SIGNAL(reloadFinished()),
-            this, SLOT(showLauncherMessage()));
-    connect(m_appBackend, SIGNAL(startingFirstTransaction()),
-            this, SLOT(addProgressItem()));
-    connect(m_appBackend, SIGNAL(sourcesEditorFinished()),
-            this, SLOT(sourcesEditorFinished()));
+    m_launches = new LaunchListModel(this);
 
-    // Other backends
-    QList<AbstractResourcesBackend*> backends;
-
-#ifdef ATTICA_ENABLED
-    backends += new KNSBackend("comic.knsrc", "face-smile-big", this);
-    backends += new KNSBackend("plasmoids.knsrc", "plasma", this);
-#endif
+    MuonBackendsFactory f;
+    QList<AbstractResourcesBackend*> backends = f.allBackends();
 
+    //TODO: should add the appBackend here too
     for (AbstractResourcesBackend *backend : backends) {
         resourcesModel->addResourcesBackend(backend);
+        backend->integrateMainWindow(this);
+        
+        if(backend->metaObject()->className()==QLatin1String("ApplicationBackend")) {
+            m_appBackend = backend;
+            connect(m_appBackend, SIGNAL(backendReady()),
+                    this, SLOT(populateViews()));
+            connect(m_appBackend, SIGNAL(reloadStarted()), //TODO: use ResourcesModel signals
+                    this, SLOT(removeProgressItem()));
+            connect(m_appBackend, SIGNAL(reloadFinished()),
+                    this, SLOT(showLauncherMessage()));
+            connect(m_appBackend, SIGNAL(startingFirstTransaction()),
+                    this, SLOT(addProgressItem()));
+            connect(m_appBackend, SIGNAL(sourcesEditorFinished()),
+                    this, SLOT(sourcesEditorFinished()));
+        }
     }
-
-    setActionsEnabled(true);
 }
 
 void MainWindow::loadSplitterSizes()
@@ -208,49 +206,86 @@ void MainWindow::clearViews()
     m_viewModel->clear();
 }
 
+QStandardItem* createOriginItem(const QString& originName, const QString& originLabel)
+{
+    // We must spread the word of Origin. Hallowed are the Ori! ;P
+    QStandardItem *viewItem = new QStandardItem;
+    viewItem->setEditable(false);
+    viewItem->setText(originLabel);
+    viewItem->setData(originName, OriginFilterRole);
+    viewItem->setData(AppView, ViewTypeRole);
+
+    if (originName == "Ubuntu") {
+        viewItem->setText(i18nc("@item:inlistbox", "Provided by Kubuntu"));
+        viewItem->setIcon(KIcon("ubuntu-logo"));
+    }
+
+    if (originName == "Debian") {
+        viewItem->setText(i18nc("@item:inlistbox", "Provided by Debian"));
+        viewItem->setIcon(KIcon("emblem-debian"));
+    }
+
+    if (originName == "Canonical") {
+        viewItem->setText(i18nc("@item:inlistbox The name of the repository provided by Canonical, Ltd. ",
+                                "Canonical Partners"));
+        viewItem->setIcon(KIcon("partner"));
+    }
+
+    if (originName.startsWith(QLatin1String("LP-PPA"))) {
+        viewItem->setIcon(KIcon("user-identity"));
+
+        if (originName == QLatin1String("LP-PPA-app-review-board")) {
+            viewItem->setText(i18nc("@item:inlistbox An independent software source",
+                                    "Independent"));
+            viewItem->setIcon(KIcon("system-users"));
+        } else
+            viewItem->setIcon(KIcon("user-identity"));
+    }
+    return viewItem;
+}
+
+bool repositoryNameLessThan(const QString& a, const QString& b)
+{
+    static QStringList prioritary(QStringList() << "Debian" << "Ubuntu" << "Canonical" << "LP-PPA-app-review-board");
+    int idxA = prioritary.indexOf(a), idxB = prioritary.indexOf(b);
+    if(idxA == idxB)
+        return a<b;
+    else if((idxB == -1) ^ (idxA == -1))
+        return idxB == -1;
+    else
+        return idxA<idxB;
+}
+
+QPair<QStringList, QStringList> fetchOrigins()
+{
+    QSet<QString> originSet, instOriginSet;
+    
+    ResourcesModel *resourcesModel = ResourcesModel::global();
+    for(int i=0; i<resourcesModel->rowCount(); i++) {
+        AbstractResource* app = resourcesModel->resourceAt(i);
+        if (app->isInstalled())
+            instOriginSet << app->origin();
+        else
+            originSet << app->origin();
+    }
+
+    originSet.remove(QString());
+    instOriginSet.remove(QString());
+    originSet += instOriginSet;
+    
+    QStringList originList=originSet.toList(), instOriginList=instOriginSet.toList();
+    qSort(originList.begin(), originList.end(), repositoryNameLessThan);
+    qSort(instOriginList.begin(), instOriginList.end(), repositoryNameLessThan);
+    return qMakePair(originList, instOriginList);
+}
+
 void MainWindow::populateViews()
 {
-    ResourcesModel *resourcesModel = ResourcesModel::global();
-    resourcesModel->addResourcesBackend(m_appBackend);
-    setCanExit(true); // APT is done reloading at this point
-    QStringList originNames = m_appBackend->appOrigins().toList();
-    QStringList originLabels;
-    QApt::Backend* backend = m_appBackend->backend();
-    foreach (const QString &originName, originNames) {
-        originLabels << backend->originLabel(originName);
-    }
-
-    if (originNames.contains("Ubuntu")) {
-        int index = originNames.indexOf("Ubuntu");
-        originNames.move(index, 0); // Move to front of the list
-
-        if (originNames.contains("Canonical")) {
-            int index = originNames.indexOf("Canonical");
-            if (originNames.size() >= 2)
-                originNames.move(index, 1); // Move to 2nd spot
-        }
-
-        if (originNames.contains("LP-PPA-app-review-board")) {
-            int index = originNames.indexOf("LP-PPA-app-review-board");
-            if (originNames.size() >= 3)
-                originNames.move(index, 2); // Move to third spot
-        }
-    }
-
-    if (originNames.contains("Debian")) {
-        int index = originNames.indexOf("Debian");
-        originNames.move(index, 0); // Move to front of the list
-    }
-
-    QStandardItem *parentItem = m_viewModel->invisibleRootItem();
-
     QStandardItem *availableItem = new QStandardItem;
     availableItem->setEditable(false);
     availableItem->setIcon(KIcon("applications-other").pixmap(32,32));
     availableItem->setText(i18nc("@item:inlistbox Parent item for available software", "Get Software"));
     availableItem->setData(CatView, ViewTypeRole);
-    parentItem->appendRow(availableItem);
-    m_viewHash[availableItem->index()] = 0;
 
     QStandardItem *installedItem = new QStandardItem;
     installedItem->setEditable(false);
@@ -258,120 +293,31 @@ void MainWindow::populateViews()
     installedItem->setText(i18nc("@item:inlistbox Parent item for installed software", "Installed Software"));
     installedItem->setData(AppView, ViewTypeRole);
     installedItem->setData(AbstractResource::State::Installed, StateFilterRole);
-    parentItem->appendRow(installedItem);
-    m_viewHash[installedItem->index()] = 0;
-
-    parentItem = availableItem;
+    
+    QPair< QStringList, QStringList > origins = fetchOrigins();
+    QStringList originNames = origins.first;
+    QApt::Backend* backend = qobject_cast<QApt::Backend*>(m_appBackend->property("backend").value<QObject*>());
     foreach(const QString &originName, originNames) {
-        QString originLabel = backend->originLabel(originName);
-        QStandardItem *viewItem = new QStandardItem;
-        viewItem->setEditable(false);
-        viewItem->setText(originLabel);
-        viewItem->setData(originName, OriginFilterRole);
-        viewItem->setData(AppView, ViewTypeRole);
-
-        if (originName == "Ubuntu") {
-            viewItem->setText(i18nc("@item:inlistbox", "Provided by Kubuntu"));
-            viewItem->setIcon(KIcon("ubuntu-logo"));
-        }
-
-        if (originName == "Debian") {
-            viewItem->setText(i18nc("@item:inlistbox", "Provided by Debian"));
-            viewItem->setIcon(KIcon("emblem-debian"));
-        }
-
-        if (originName == "Canonical") {
-            viewItem->setText(i18nc("@item:inlistbox The name of the repository provided by Canonical, Ltd. ",
-                                    "Canonical Partners"));
-            viewItem->setIcon(KIcon("partner"));
-        }
-
-        if (originName.startsWith(QLatin1String("LP-PPA"))) {
-            viewItem->setIcon(KIcon("user-identity"));
-
-            if (originName == QLatin1String("LP-PPA-app-review-board")) {
-                viewItem->setText(i18nc("@item:inlistbox An independent software source",
-                                        "Independent"));
-                viewItem->setIcon(KIcon("system-users"));
-            }
-        }
-
-        availableItem->appendRow(viewItem);
-        m_viewHash[viewItem->index()] = 0;
+        availableItem->appendRow(createOriginItem(originName, backend->originLabel(originName)));
     }
 
-    QStringList instOriginNames = m_appBackend->installedAppOrigins().toList();
-    QStringList instOriginLabels;
-    foreach (const QString &originName, instOriginNames) {
-        instOriginLabels << backend->originLabel(originName);
-    }
-
-    if (instOriginNames.contains("Ubuntu")) {
-        int index = instOriginNames.indexOf("Ubuntu");
-        instOriginNames.move(index, 0); // Move to front of the list
-
-        if (instOriginNames.contains("Canonical")) {
-            int index = instOriginNames.indexOf("Canonical");
-            if (originNames.size() >= 2)
-                instOriginNames.move(index, 1); // Move to 2nd spot
-        }
-
-        if (instOriginNames.contains("LP-PPA-app-review-board")) {
-            int index = instOriginNames.indexOf("LP-PPA-app-review-board");
-            if (originNames.size() >= 3)
-                originNames.move(index, 2); // Move to third spot
-        }
-    }
-
-    if (instOriginNames.contains("Debian")) {
-        int index = instOriginNames.indexOf("Debian");
-        instOriginNames.move(index, 0); // Move to front of the list
-    }
-
-    parentItem = installedItem;
+    QStringList instOriginNames = origins.second;
     foreach(const QString & originName, instOriginNames) {
-        // We must spread the word of Origin. Hallowed are the Ori! ;P
-        QString originLabel = backend->originLabel(originName);
-        QStandardItem *viewItem = new QStandardItem;
-        viewItem->setEditable(false);
-        viewItem->setText(originLabel);
+        QStandardItem* viewItem = createOriginItem(originName, backend->originLabel(originName));
+
         viewItem->setData(AbstractResource::State::Installed, StateFilterRole);
-        viewItem->setData(originName, OriginFilterRole);
-        viewItem->setData(AppView, ViewTypeRole);
-
-        if (originName == "Ubuntu") {
-            viewItem->setText(i18nc("@item:inlistbox", "Provided by Kubuntu"));
-            viewItem->setIcon(KIcon("ubuntu-logo"));
-        }
-
-        if (originName == "Canonical") {
-            viewItem->setText(i18nc("@item:inlistbox The name of the repository provided by Canonical, Ltd. ",
-                                    "Canonical Partners"));
-            viewItem->setIcon(KIcon("partner"));
-        }
-
-        if (originName.startsWith(QLatin1String("LP-PPA"))) {
-            if (originName == QLatin1String("LP-PPA-app-review-board")) {
-                viewItem->setText(i18nc("@item:inlistbox An independent software source",
-                                        "Independent"));
-            }
-            viewItem->setIcon(KIcon("user-identity"));
-        }
-
         installedItem->appendRow(viewItem);
-        m_viewHash[viewItem->index()] = 0;
     }
 
-    parentItem = m_viewModel->invisibleRootItem();
-
     QStandardItem *historyItem = new QStandardItem;
     historyItem->setEditable(false);
     historyItem->setIcon(KIcon("view-history").pixmap(32,32));
     historyItem->setText(i18nc("@item:inlistbox Item for showing the history view", "History"));
     historyItem->setData(History, ViewTypeRole);
-    parentItem->appendRow(historyItem);
-    m_viewHash[historyItem->index()] = 0;
 
+    m_viewModel->appendRow(availableItem);
+    m_viewModel->appendRow(installedItem);
+    m_viewModel->appendRow(historyItem);
     selectFirstRow(m_viewSwitcher);
 }
 
@@ -395,7 +341,7 @@ void MainWindow::changeView(const QModelIndex &index)
             if (originFilter != QLatin1String("Ubuntu") && originFilter != QLatin1String("Debian"))
                 appView->setShouldShowTechnical(true);
         }
-        break;
+            break;
         case CatView:
             view = new AvailableView(this);
             break;
@@ -429,13 +375,6 @@ void MainWindow::selectFirstRow(const QAbstractItemView *itemView)
     changeView(firstRow);
 }
 
-void MainWindow::runSourcesEditor()
-{
-    // Let QApt Batch handle the update GUI
-    // FIXME?
-    //MuonMainWindow::runSourcesEditor(true);
-}
-
 void MainWindow::sourcesEditorFinished()
 {
     clearViews();
@@ -447,32 +386,18 @@ void MainWindow::showLauncherMessage()
 {
     clearMessageActions();
 
-    QVector<KService::Ptr> apps;
-    foreach (Application *app, m_appBackend->launchList()) {
-        app->clearPackage();
-        app->package(); // Regenerate package
-        if (!app->isInstalled()) {
-            continue;
-        }
-        apps << app->executables();
-    }
-
-    m_appBackend->clearLaunchList();
-    m_launchableApps = apps;
-
-    if (apps.size() == 1) {
-        KService::Ptr service = apps.first();
-        QString name = service->genericName().isEmpty() ?
-                       service->property("Name").toString() :
-                       service->property("Name").toString() % QLatin1Literal(" - ") % service->genericName();
+    if (m_launches->rowCount()==1) {
+        QModelIndex index = m_launches->index(0, 0);
+        QString name = index.data().toString();
         m_launcherMessage->setText(i18nc("@info", "%1 was successfully installed.", name));
 
-        KAction *launchAction = new KAction(KIcon(service->icon()), i18nc("@action", "Start"), this);
+        KIcon launchIcon = KIcon(index.data(Qt::DecorationRole).value<QIcon>());
+        KAction *launchAction = new KAction(launchIcon, i18nc("@action", "Start"), this);
         connect(launchAction, SIGNAL(activated()), this, SLOT(launchSingleApp()));
 
         m_launcherMessage->addAction(launchAction);
         m_launcherMessage->animatedShow();
-    } else if (apps.size() > 1) {
+    } else if (m_launches->rowCount() > 1) {
         m_launcherMessage->setText(i18nc("@info", "Applications successfully installed."));
         KAction *launchAction = new KAction(i18nc("@action", "Run New Applications..."), this);
         connect(launchAction, SIGNAL(activated()), this, SLOT(showAppLauncher()));
@@ -483,19 +408,19 @@ void MainWindow::showLauncherMessage()
 
 void MainWindow::launchSingleApp()
 {
-    KToolInvocation::startServiceByDesktopPath(m_launchableApps.first()->desktopEntryPath());
+    m_launches->invokeApplication(0);
     m_launcherMessage->animatedHide();
     m_launcherMessage->removeAction(m_launcherMessage->actions().first());
 }
 
 void MainWindow::showAppLauncher()
 {
-    if (!m_appLauncher && !m_launchableApps.isEmpty()) {
-        m_appLauncher = new ApplicationLauncher(m_appBackend);
+    if (!m_appLauncher && !m_launches->rowCount()==0) {
+        m_appLauncher = new ApplicationLauncher(m_launches);
         connect(m_appLauncher, SIGNAL(destroyed(QObject*)),
-            this, SLOT(onAppLauncherClosed()));
+                this, SLOT(onAppLauncherClosed()));
         connect(m_appLauncher, SIGNAL(finished(int)),
-            this, SLOT(onAppLauncherClosed()));
+                this, SLOT(onAppLauncherClosed()));
         m_appLauncher->setWindowTitle(i18nc("@title:window", "Installation Complete"));
         m_appLauncher->show();
     }
@@ -550,11 +475,10 @@ void MainWindow::removeProgressItem()
     if (!m_progressItem)
         return;
 
-    QObject *progressView = m_viewHash[m_progressItem->index()];
+    QObject *progressView = m_viewHash.take(m_progressItem->index());
     if (progressView)
-            progressView->deleteLater();
+        progressView->deleteLater();
 
-    m_viewHash.remove(m_progressItem->index());
-    m_viewModel->removeRow(m_viewModel->indexFromItem(m_progressItem).row());
+    m_viewModel->removeRow(m_progressItem->row());
     m_progressItem = nullptr;
 }
