@@ -24,12 +24,11 @@
 // KDE includes
 #include <KService>
 
-// LibQApt includes
-#include <LibQApt/Backend>
-
 // Own includes
 #include "../libmuon/MuonMainWindow.h"
 
+class AbstractResourcesBackend;
+class LaunchListModel;
 class QAbstractItemView;
 class QModelIndex;
 class QSplitter;
@@ -55,7 +54,7 @@ public:
     ~MainWindow();
 
 private:
-    ApplicationBackend *m_appBackend;
+    AbstractResourcesBackend *m_appBackend;
     QSplitter *m_mainWidget;
     QStackedWidget *m_viewStack;
     QWidget *m_busyWidget;
@@ -64,12 +63,12 @@ private:
     QHash<QModelIndex, QWidget *> m_viewHash;
     KAction *m_loadSelectionsAction;
     KAction *m_saveSelectionsAction;
+    LaunchListModel *m_launches;
     KMessageWidget *m_launcherMessage;
     ApplicationLauncher *m_appLauncher;
     ProgressView *m_progressView;
     QStandardItem *m_progressItem;
 
-    QVector<KService::Ptr> m_launchableApps;
     int m_transactionCount;
 
 private Q_SLOTS:
@@ -81,7 +80,6 @@ private Q_SLOTS:
     void populateViews();
     void changeView(const QModelIndex &index);
     void selectFirstRow(const QAbstractItemView *itemView);
-    void runSourcesEditor();
     void sourcesEditorFinished();
     void showLauncherMessage();
     void launchSingleApp();
