@@ -29,7 +29,9 @@
 #include <Transaction/Transaction.h>
 #include <QDebug>
 
-K_GLOBAL_STATIC_WITH_ARGS(ResourcesModel, globalResourcesModel, (0))
+static const KCatalogLoader loader("libmuon");
+
+K_GLOBAL_STATIC(ResourcesModel, globalResourcesModel)
 
 ResourcesModel *ResourcesModel::global()
 {
@@ -175,11 +177,14 @@ QVariant ResourcesModel::data(const QModelIndex& index, int role) const
 
 int ResourcesModel::rowCount(const QModelIndex& parent) const
 {
-    if(parent.isValid()) return 0;
+    if(parent.isValid())
+        return 0; // Not the root element, and children don't have subchildren
+
+    // The root element parents all resources from all backends
     int ret = 0;
-    foreach(const QVector<AbstractResource*>& resources, m_resources) {
+    for (const QVector<AbstractResource*>& resources : m_resources)
         ret += resources.size();
-    }
+
     return ret;
 }
 
@@ -235,7 +240,7 @@ void ResourcesModel::updateCaller()
     int pos = m_backends.indexOf(backend);
     QVector<AbstractResource*>* backendsResources = &m_resources[pos];
     int before = 0;
-    for(QVector<QVector<AbstractResource*> >::const_iterator it=m_resources.constBegin();
+    for(auto it=m_resources.constBegin();
         it!=m_resources.constEnd() && it!=backendsResources;
         ++it)
     {
@@ -303,7 +308,7 @@ QMap<int, QVariant> ResourcesModel::itemData(const QModelIndex& index) const
 {
     QMap<int, QVariant> ret;
     QHash<int, QByteArray> names = roleNames();
-    for(QHash<int, QByteArray>::const_iterator it=names.constBegin(), itEnd=names.constEnd(); it!=itEnd; ++it) {
+    for (auto it = names.constBegin(); it != names.constEnd(); ++it) {
         ret.insert(it.key(), data(index, it.key()));
     }
     return ret;
