@@ -42,9 +42,11 @@
 #include <LibQApt/Transaction>
 
 // Own includes
-#include "../libmuon/HistoryView/HistoryView.h"
-#include "../libmuon/MuonStrings.h"
-#include "../libmuon/QAptActions.h"
+#include <MuonBackendsFactory.h>
+#include <resources/AbstractResourcesBackend.h>
+#include "../libmuonapt/HistoryView/HistoryView.h"
+#include "../libmuonapt/MuonStrings.h"
+#include "../libmuonapt/QAptActions.h"
 #include "ChangelogWidget.h"
 #include "ProgressWidget.h"
 #include "config/UpdaterSettingsDialog.h"
@@ -56,8 +58,11 @@ MainWindow::MainWindow()
     , m_historyDialog(nullptr)
     , m_checkerProcess(nullptr)
 {
+    MuonBackendsFactory f;
+    m_apps = f.backend("muon-appsbackend");
+    connect(m_apps, SIGNAL(backendReady()), SLOT(initBackend()));
+
     initGUI();
-    QTimer::singleShot(10, this, SLOT(initObject()));
 }
 
 void MainWindow::initGUI()
@@ -84,14 +89,11 @@ void MainWindow::initGUI()
     m_progressWidget->hide();
 
     m_updaterWidget = new UpdaterWidget(mainWidget);
-    connect(this, SIGNAL(backendReady(QApt::Backend*)),
-            m_updaterWidget, SLOT(setBackend(QApt::Backend*)));
+    m_updaterWidget->setEnabled(false);
 
     m_changelogWidget = new ChangelogWidget(this);
     m_changelogWidget->hide();
-    connect(this, SIGNAL(backendReady(QApt::Backend*)),
-            m_changelogWidget, SLOT(setBackend(QApt::Backend*)));
-    connect(m_updaterWidget, SIGNAL(packageChanged(QApt::Package*)),
+    connect(m_updaterWidget, SIGNAL(selectedPackageChanged(QApt::Package*)),
             m_changelogWidget, SLOT(setPackage(QApt::Package*)));
 
     mainLayout->addWidget(m_powerMessage);
@@ -100,79 +102,19 @@ void MainWindow::initGUI()
     mainLayout->addWidget(m_updaterWidget);
     mainLayout->addWidget(m_changelogWidget);
 
-    m_backend = new QApt::Backend(this);
-    m_actions->setBackend(m_backend);
-    connect(m_actions, SIGNAL(checkForUpdates()),
-            this, SLOT(checkForUpdates()));
-
+    m_apps->integrateMainWindow(this);
     setupActions();
-
     mainWidget->setLayout(mainLayout);
     setCentralWidget(mainWidget);
 
     checkDistUpgrade();
 }
 
-void MainWindow::initObject()
-{
-    if (!m_backend->init())
-        initError();
-
-    if (m_backend->xapianIndexNeedsUpdate()) {
-        m_backend->updateXapianIndex();
-    }
-
-    emit backendReady(m_backend);
-    setCanExit(true);
-
-    setActionsEnabled(); //Get initial enabled/disabled state
-}
-
-void MainWindow::initError()
-{
-    QString details = m_backend->initErrorMessage();
-
-    MuonStrings *muonStrings = MuonStrings::global();
-
-    QString title = muonStrings->errorTitle(QApt::InitError);
-    QString text = muonStrings->errorText(QApt::InitError, nullptr);
-
-    KMessageBox::detailedError(this, text, details, title);
-    exit(-1);
-}
-
 void MainWindow::setupActions()
 {
     MuonMainWindow::setupActions();
 
-    m_loadSelectionsAction = actionCollection()->addAction("open_markings");
-    m_loadSelectionsAction->setIcon(KIcon("document-open"));
-    m_loadSelectionsAction->setText(i18nc("@action", "Read Markings..."));
-    connect(m_loadSelectionsAction, SIGNAL(triggered()), m_actions, SLOT(loadSelections()));
-
-    m_saveSelectionsAction = actionCollection()->addAction("save_markings");
-    m_saveSelectionsAction->setIcon(KIcon("document-save-as"));
-    m_saveSelectionsAction->setText(i18nc("@action", "Save Markings As..."));
-    connect(m_saveSelectionsAction, SIGNAL(triggered()), m_actions, SLOT(saveSelections()));
-
-    m_createDownloadListAction = actionCollection()->addAction("save_download_list");
-    m_createDownloadListAction->setIcon(KIcon("document-save-as"));
-    m_createDownloadListAction->setText(i18nc("@action", "Save Package Download List..."));
-    connect(m_createDownloadListAction, SIGNAL(triggered()), m_actions, SLOT(createDownloadList()));
-
-    m_downloadListAction = actionCollection()->addAction("download_from_list");
-    m_downloadListAction->setIcon(KIcon("download"));
-    m_downloadListAction->setText(i18nc("@action", "Download Packages From List..."));
-    connect(m_downloadListAction, SIGNAL(triggered()), m_actions, SLOT(downloadPackagesFromList()));
-    if (!m_actions->isConnected()) {
-        m_downloadListAction->setDisabled(false);
-    }
-    connect(m_actions, SIGNAL(shouldConnect(bool)), m_downloadListAction, SLOT(setEnabled(bool)));
-
-    m_loadArchivesAction = actionCollection()->addAction("load_archives");
-    m_loadArchivesAction->setIcon(KIcon("document-open"));
-    m_loadArchivesAction->setText(i18nc("@action", "Add Downloaded Packages"));
-    connect(m_loadArchivesAction, SIGNAL(triggered()), m_actions, SLOT(loadArchives()));
+    connect(QAptActions::self(), SIGNAL(checkForUpdates()), this, SLOT(checkForUpdates()));
 
     m_applyAction = actionCollection()->addAction("apply");
     m_applyAction->setIcon(KIcon("dialog-ok-apply"));
@@ -197,6 +139,14 @@ void MainWindow::setupActions()
     setupGUI((StandardWindowOption)(KXmlGuiWindow::Default & ~KXmlGuiWindow::StatusBar));
 }
 
+void MainWindow::initBackend()
+{
+    m_updaterWidget->setBackend(m_apps);
+    m_updaterWidget->setEnabled(true);
+
+    setActionsEnabled();
+}
+
 void MainWindow::transactionStatusChanged(QApt::TransactionStatus status)
 {
     // FIXME: better support for transactions that do/don't need reloads
@@ -239,12 +189,12 @@ void MainWindow::reload()
     setCanExit(false);
     m_changelogWidget->stopPendingJobs();
 
-    disconnect(m_updaterWidget, SIGNAL(packageChanged(QApt::Package*)),
+    disconnect(m_updaterWidget, SIGNAL(selectedPackageChanged(QApt::Package*)),
                m_changelogWidget, SLOT(setPackage(QApt::Package*)));
 
     m_updaterWidget->reload();
 
-    connect(m_updaterWidget, SIGNAL(packageChanged(QApt::Package*)),
+    connect(m_updaterWidget, SIGNAL(selectedPackageChanged(QApt::Package*)),
             m_changelogWidget, SLOT(setPackage(QApt::Package*)));
 
     m_changelogWidget->setPackage(0);
@@ -257,14 +207,12 @@ void MainWindow::reload()
 
 void MainWindow::setActionsEnabled(bool enabled)
 {
-    m_actions->setActionsEnabled(enabled);
+    QAptActions::self()->setActionsEnabled(enabled);
     if (!enabled) {
         return;
     }
 
-    m_downloadListAction->setEnabled(m_actions->isConnected());
-
-    m_applyAction->setEnabled(m_backend->areChangesMarked());
+    m_applyAction->setEnabled(backend()->areChangesMarked());
     m_updaterWidget->setEnabled(true);
 }
 
@@ -276,7 +224,7 @@ void MainWindow::checkForUpdates()
     m_changelogWidget->animatedHide();
     m_changelogWidget->stopPendingJobs();
 
-    m_trans = m_backend->updateCache();
+    m_trans = backend()->updateCache();
     setupTransaction(m_trans);
     m_trans->run();
 }
@@ -289,7 +237,7 @@ void MainWindow::startCommit()
     m_changelogWidget->animatedHide();
     m_changelogWidget->stopPendingJobs();
 
-    m_trans = m_backend->commitChanges();
+    m_trans = backend()->commitChanges();
     setupTransaction(m_trans);
     m_trans->run();
 }
@@ -381,7 +329,7 @@ void MainWindow::checkPlugState()
 
 void MainWindow::updatePlugState(bool plugged)
 {
-    plugged ? m_powerMessage->hide() : m_powerMessage->show();
+    m_powerMessage->setVisible(!plugged);
 }
 
 void MainWindow::checkDistUpgrade()
@@ -409,3 +357,8 @@ void MainWindow::launchDistUpgrade()
     KProcess::startDetached(QStringList() << "python"
                             << "/usr/share/pyshared/UpdateManager/DistUpgradeFetcherKDE.py");
 }
+
+QApt::Backend* MainWindow::backend() const
+{
+    return qobject_cast<QApt::Backend*>(m_apps->property("backend").value<QObject*>());
+}
