@@ -28,6 +28,7 @@
 #include <QtGui/QLabel>
 #include <QtGui/QTreeView>
 #include <QtGui/QVBoxLayout>
+#include <qaction.h>
 
 // KDE includes
 #include <KIcon>
@@ -36,13 +37,17 @@
 #include <KPixmapSequence>
 #include <KPixmapSequenceOverlayPainter>
 #include <KDebug>
+#include <KMessageWidget>
 
 // LibQApt includes
 #include <LibQApt/Backend>
 
 // Libmuon includes
-#include <ApplicationBackend/Application.h>
-#include <ChangesDialog.h>
+#include <resources/AbstractResourcesBackend.h>
+#include <resources/AbstractResource.h>
+
+//libmuonapt includes
+#include "../libmuonapt/ChangesDialog.h"
 
 // Own includes
 #include "UpdateModel/UpdateModel.h"
@@ -61,18 +66,30 @@ UpdaterWidget::UpdaterWidget(QWidget *parent) :
     page1Layout->setSpacing(0);
     page1->setLayout(page1Layout);
 
+    QString text = i18nc("@label", "<em>Some packages were not marked for update.</em><p/>"
+                            "The update of these packages need some others to be installed or removed.<p/>"
+                            "Do you want to update those too?");
+    QAction* action = new QAction(KIcon("dialog-ok-apply"), i18n("Mark All"), this);
+    connect(action, SIGNAL(triggered(bool)), SLOT(markAllPackagesForUpgrade()));
+    m_upgradesWidget = new KMessageWidget(this);
+    m_upgradesWidget->setText(text);
+    m_upgradesWidget->addAction(action);
+    m_upgradesWidget->setCloseButtonVisible(true);
+    m_upgradesWidget->setVisible(false);
+    page1Layout->addWidget(m_upgradesWidget);
+
     m_updateModel = new UpdateModel(page1);
 
-    connect(m_updateModel, SIGNAL(checkApps(QList<Application*>,bool)),
-            this, SLOT(checkApps(QList<Application*>,bool)));
+    connect(m_updateModel, SIGNAL(checkApps(QList<AbstractResource*>,bool)),
+            this, SLOT(checkApps(QList<AbstractResource*>,bool)));
 
     m_updateView = new QTreeView(page1);
     m_updateView->setAlternatingRowColors(true);
+    m_updateView->setModel(m_updateModel);
     m_updateView->header()->setResizeMode(0, QHeaderView::Stretch);
     m_updateView->header()->setResizeMode(1, QHeaderView::ResizeToContents);
     m_updateView->header()->setResizeMode(2, QHeaderView::ResizeToContents);
     m_updateView->header()->setStretchLastSection(false);
-    m_updateView->setModel(m_updateModel);
     connect(m_updateView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
             this, SLOT(selectionChanged(QItemSelection,QItemSelection)));
     page1Layout->addWidget(m_updateView);
@@ -115,9 +132,10 @@ UpdaterWidget::UpdaterWidget(QWidget *parent) :
     setCurrentWidget(page1);
 }
 
-void UpdaterWidget::setBackend(QApt::Backend *backend)
+void UpdaterWidget::setBackend(AbstractResourcesBackend *backend)
 {
-    m_backend = backend;
+    m_appsBackend = backend;
+    m_backend = qobject_cast<QApt::Backend*>(backend->property("backend").value<QObject*>());
     connect(m_backend, SIGNAL(packageChanged()),
             m_updateModel, SLOT(packageChanged()));
 
@@ -127,7 +145,7 @@ void UpdaterWidget::setBackend(QApt::Backend *backend)
 void UpdaterWidget::reload()
 {
     m_updateModel->clear();
-    m_backend->reloadCache();
+    QMetaObject::invokeMethod(m_appsBackend, "reload");
 
     setCurrentIndex(0);
     populateUpdateModel();
@@ -135,9 +153,7 @@ void UpdaterWidget::reload()
 
 void UpdaterWidget::populateUpdateModel()
 {
-    QApt::PackageList upgradeList = m_backend->upgradeablePackages();
-
-    if (upgradeList.isEmpty()) {
+    if (m_appsBackend->updatesCount()==0) {
         QApplication::restoreOverrideCursor();
         m_busyWidget->stop();
         checkUpToDate();
@@ -152,69 +168,40 @@ void UpdaterWidget::populateUpdateModel()
 
     UpdateItem *systemItem = new UpdateItem(i18nc("@item:inlistbox", "System Updates"),
                                              KIcon("applications-system"));
+    
+    QVector<AbstractResource*> resources = m_appsBackend->allResources();
+    foreach(AbstractResource* res, resources) {
+        if(res->state()==AbstractResource::Upgradeable) {
+            UpdateItem *updateItem = new UpdateItem(res);
+            QApt::Package* package = retrievePackage(res);
 
-    QDir appDir("/usr/share/app-install/desktop/");
-    QStringList fileList = appDir.entryList(QDir::Files);
-
-    foreach(const QString &fileName, fileList) {
-        if (fileName.endsWith(QLatin1String(".menu"))) // Skip non-desktop files that can slip in
-            continue;
-        Application *app = new Application("/usr/share/app-install/desktop/" + fileName, m_backend);
-        QApt::Package *package = app->package();
-        if (!package || !upgradeList.contains(package)) {
-            delete app;
-            continue;
-        }
-        int state = package->state();
-
-        if (!(state & QApt::Package::Upgradeable)) {
-            delete app;
-            continue;
-        }
-
-        UpdateItem *updateItem = new UpdateItem(app);
-
-        // Set update type
-        bool securityFound = false;
-        for (const QString &archive : package->archives()) {
-            if (archive.contains(QLatin1String("security"))) {
-                securityFound = true;
-                break;
+            if (!package) {
+                kDebug() << "No package:" << res->name();
+                continue;
+            }
+            
+            bool securityFound = false;
+            for (const QString &archive : package->archives()) {
+                if (archive.contains(QLatin1String("security"))) {
+                    securityFound = true;
+                    break;
+                }
+            }
+            
+            if(!res->isTechnical()) {
+                if (securityFound) {
+                    securityItem->appendChild(updateItem);
+                } else {
+                    appItem->appendChild(updateItem);
+                }
+            } else {
+                if (securityFound) {
+                    securityItem->appendChild(updateItem);
+                } else {
+                    systemItem->appendChild(updateItem);
+                }
             }
         }
-
-        if (securityFound) {
-            securityItem->appendChild(updateItem);
-        } else {
-            appItem->appendChild(updateItem);
-        }
-
-        m_upgradeableApps.append(app);
-
-        upgradeList.removeAll(package);
-    }
-
-    // Remaining packages in the upgrade list aren't applications
-    foreach (QApt::Package *package, upgradeList) {
-        Application *app = new Application(package, m_backend);
-        UpdateItem *updateItem = new UpdateItem(app);
-
-        // Set update type
-        bool securityFound = false;
-        for (const QString &archive : package->archives()) {
-            if (archive.contains(QLatin1String("security"))) {
-                securityFound = true;
-                break;
-            }
-        }
-
-        if (securityFound) {
-            securityItem->appendChild(updateItem);
-        } else {
-            systemItem->appendChild(updateItem);
-        }
-
-        m_upgradeableApps.append(app);
     }
 
     // Add populated items to the model
@@ -250,11 +237,11 @@ void UpdaterWidget::populateUpdateModel()
     checkUpToDate();
 }
 
-void UpdaterWidget::checkApps(QList<Application *> apps, bool checked)
+void UpdaterWidget::checkApps(QList<AbstractResource*> apps, bool checked)
 {
     QApt::PackageList list;
-    foreach (Application *app, apps) {
-        list << app->package();
+    foreach (AbstractResource *app, apps) {
+        list << retrievePackage(app);
     }
 
     QApt::Package::State action = checked ? QApt::Package::ToInstall : QApt::Package::ToKeep;
@@ -296,15 +283,11 @@ void UpdaterWidget::selectionChanged(const QItemSelection &selected,
     QModelIndexList indexes = selected.indexes();
     QApt::Package *package = 0;
 
-    if (indexes.isEmpty()) {
-        emit packageChanged(package);
-        return;
+    if (!indexes.isEmpty()) {
+        package = retrievePackage(m_updateModel->itemFromIndex(indexes.first())->app());
     }
 
-    Application *app = m_updateModel->itemFromIndex(indexes.first())->app();
-    package = app ? app->package() : 0;
-
-    emit packageChanged(package);
+    emit selectedPackageChanged(package);
 }
 
 void UpdaterWidget::checkAllMarked()
@@ -312,30 +295,21 @@ void UpdaterWidget::checkAllMarked()
     QApt::PackageList upgradeable = m_backend->upgradeablePackages();
     int markedCount = m_backend->packageCount(QApt::Package::ToUpgrade);
 
-    if (markedCount < upgradeable.count())
-    {
-        QString text = i18nc("@label", "Not all packages could be marked for upgrade. "
-                             "The available upgrades may require new packages to "
-                             "be installed or removed. Do you want to mark upgrades "
-                             "that may require the installation or removal of additional "
-                             "packages?");
-        QString title = i18nc("@title:window", "Unable to Mark Upgrades");
-        KGuiItem markUpgrades(i18nc("@action", "Mark Upgrades"));
+    m_upgradesWidget->setVisible(markedCount < upgradeable.count());
+}
 
-        int res = KMessageBox::questionYesNo(this, text, title, markUpgrades);
+void UpdaterWidget::markAllPackagesForUpgrade()
+{
+    QApt::PackageList upgradeable = m_backend->upgradeablePackages();
 
-        if (res != KMessageBox::Yes)
-            return;
+    // Mark dist upgrade
+    m_oldCacheState = m_backend->currentCacheState();
+    m_backend->saveCacheState();
+    m_backend->markPackagesForDistUpgrade();
 
-        // Mark dist upgrade
-        m_oldCacheState = m_backend->currentCacheState();
-        m_backend->saveCacheState();
-        m_backend->markPackagesForDistUpgrade();
+    checkChanges(m_backend->stateChanges(m_oldCacheState, upgradeable));
 
-        // Show user packages to be installed/removed
-        auto changes = m_backend->stateChanges(m_oldCacheState, upgradeable);
-        checkChanges(changes);
-    }
+    m_upgradesWidget->hide();
 }
 
 void UpdaterWidget::checkUpToDate()
@@ -376,3 +350,8 @@ void UpdaterWidget::checkUpToDate()
         }
     }
 }
+
+QApt::Package* UpdaterWidget::retrievePackage(AbstractResource* res)
+{
+    return res ? m_backend->package(res->packageName()) : 0;
+}
